#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "common.h"

static void
printhex(unsigned char *b, int len)
{
	int i;

	printf("hex: ");
	for (i = 0; i < len; i++)
		printf("%02X", b[i]);
}

#define CHECK_STMT_RESULT2(rc, msg, hstmt)\
    if (SQL_SUCCESS != rc && SQL_SUCCESS_WITH_INFO != rc && SQL_NEED_DATA != rc) \
    { \
        print_diag(msg, SQL_HANDLE_STMT, hstmt);\
        exit(1);\
    }

int main(int argc, char **argv)
{
	int rc;
	HSTMT hstmt = SQL_NULL_HSTMT;
	char param1[20] = { 1, 2, 3, 4, 5, 6, 7, 8 };
	SQLLEN cbParam1;
	char buf[100];
	SQLLEN ind;

	test_connect();

	rc = SQLAllocHandle(SQL_HANDLE_STMT, conn, &hstmt);
	if (!SQL_SUCCEEDED(rc))
	{
		print_diag("failed to allocate stmt handle", SQL_HANDLE_DBC, conn);
		exit(1);
	}

	/**** Insert a Large Object */
	printf("inserting large object...\n");
	rc = SQLPrepare(hstmt, (SQLCHAR *) "INSERT INTO lo_test_tab VALUES (1, ?)", SQL_NTS);
	CHECK_STMT_RESULT(rc, "SQLPrepare failed", hstmt);

	/* bind LO param as a data at execution parameter */
	cbParam1 = SQL_DATA_AT_EXEC;
	rc = SQLBindParameter(hstmt, 1, SQL_PARAM_INPUT,
						  SQL_C_BINARY,	/* value type */
						  SQL_LONGVARBINARY,	/* param type */
						  0,			/* column size */
						  0,			/* dec digits */
						  param1,		/* param value ptr */
						  0,			/* buffer len */
						  &cbParam1		/* StrLen_or_IndPtr */);
	CHECK_STMT_RESULT(rc, "SQLBindParameter failed", hstmt);

	/* Execute */
	rc = SQLExecute(hstmt);
	CHECK_STMT_RESULT2(rc, "SQLExecute failed", hstmt);

    /* Check for data-at-execute parameters */
    if (SQL_NEED_DATA == rc)
    {
        SQLLEN lobByteSize = 8; // sizeof param1
        SQLLEN lobChunkSize = 4096;
        SQLPOINTER pParamId = 0;
        const char *what = 0;

        /* Get the parameter that needs data */
        what = "SQLParamData failed (first call)";
        rc = SQLParamData(hstmt, &pParamId);  
        int error = SQL_SUCCESS != rc && SQL_SUCCESS_WITH_INFO != rc && SQL_NEED_DATA != rc;
        if (SQL_NEED_DATA == rc)
        {
            /* Send parameter data in chunks */
            what = "SQLPutData failed (next chunk)";
            while (!error && lobByteSize > lobChunkSize)
            {
                rc = SQLPutData(hstmt, (SQLPOINTER)pParamId, lobChunkSize);
                lobByteSize -= lobChunkSize;
                error = SQL_SUCCESS != rc && SQL_SUCCESS_WITH_INFO != rc && SQL_NEED_DATA != rc;
            }
            
            /* Send final chunk */
            if (!error)
            {
                what = "SQLPutData failed (final chunk)";
                rc = SQLPutData(hstmt, (SQLPOINTER)pParamId, lobByteSize);
                error = SQL_SUCCESS != rc && SQL_SUCCESS_WITH_INFO != rc && SQL_NEED_DATA != rc;
            }

            /* Make final call */
            if (!error)
            {
                what = "SQLParamData failed (final call)";
                rc = SQLParamData(hstmt, &pParamId);
                error = SQL_SUCCESS != rc && SQL_SUCCESS_WITH_INFO != rc;
            }
        }
        CHECK_STMT_RESULT(rc, (char*)what, hstmt);
    }

	rc = SQLFreeStmt(hstmt, SQL_CLOSE);
	CHECK_STMT_RESULT(rc, "SQLFreeStmt failed", hstmt);

	/**** Read it back ****/
	printf("reading it back...\n");
	SQLExecDirect(hstmt, (SQLCHAR *) "SELECT id, large_data FROM lo_test_tab WHERE id = 1", SQL_NTS);

	rc = SQLFetch(hstmt);
	CHECK_STMT_RESULT(rc, "SQLFetch failed", hstmt);

	rc = SQLGetData(hstmt, 2, SQL_C_BINARY, buf, sizeof(buf), &ind);
	CHECK_STMT_RESULT(rc, "SQLGetData failed", hstmt);

	printhex(buf, ind);
	printf("\n");
	
	/* Clean up */
	test_disconnect();

	return 0;
}
